@@ -4,7 +4,6 @@ Obviously not a typical message broker, but convenient if you don't have a broke
 methods.
 """
 
-import socket
 import struct
 
 from stomp.connect import BaseConnection
@@ -12,7 +11,7 @@ from stomp.protocol import *
 from stomp.transport import *
 from stomp.utils import *
 
-MCAST_GRP = '224.1.1.1'
+MCAST_GRP = "224.1.1.1"
 MCAST_PORT = 5000
 
 
@@ -36,7 +35,7 @@ class MulticastTransport(Transport):
         self.receiver_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
         self.receiver_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self.receiver_socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
-        self.receiver_socket.bind(('', MCAST_PORT))
+        self.receiver_socket.bind(("", MCAST_PORT))
         mreq = struct.pack("4sl", socket.inet_aton(MCAST_GRP), socket.INADDR_ANY)
         self.receiver_socket.setsockopt(socket.IPPROTO_IP, socket.IP_ADD_MEMBERSHIP, mreq)
 
@@ -66,28 +65,28 @@ class MulticastTransport(Transport):
         """
         frame_type = f.cmd.lower()
 
-        if frame_type in ['disconnect']:
+        if frame_type in ["disconnect"]:
             return
 
-        if frame_type == 'send':
-            frame_type = 'message'
-            f.cmd = 'MESSAGE'
+        if frame_type == "send":
+            frame_type = "message"
+            f.cmd = "MESSAGE"
 
-        if frame_type in ['connected', 'message', 'receipt', 'error', 'heartbeat']:
-            if frame_type == 'message':
-                if f.headers['destination'] not in self.subscriptions.values():
+        if frame_type in ["connected", "message", "receipt", "error", "heartbeat"]:
+            if frame_type == "message":
+                if f.headers["destination"] not in self.subscriptions.values():
                     return
-                (f.headers, f.body) = self.notify('before_message', f.headers, f.body)
-            self.notify(frame_type, f.headers, f.body)
-        if 'receipt' in f.headers:
-            receipt_frame = Frame('RECEIPT', {'receipt-id': f.headers['receipt']})
+                self.notify("before_message", f)
+            self.notify(frame_type, f)
+        if "receipt" in f.headers:
+            receipt_frame = Frame("RECEIPT", {"receipt-id": f.headers["receipt"]})
             lines = convert_frame(receipt_frame)
             self.send(encode(pack(lines)))
-        log.debug("Received frame: %r, headers=%r, body=%r", f.cmd, f.headers, f.body)
+        logging.debug("Received frame: %r, headers=%r, body=%r", f.cmd, f.headers, f.body)
 
     def stop(self):
         self.running = False
-        if hasattr(self.receiver_socket, 'SHUT_RDWR'):
+        if hasattr(self.receiver_socket, "SHUT_RDWR"):
             self.receiver_socket.shutdown(socket.SHUT_RDWR)
         self.receiver_socket.close()
         self.disconnect_socket()
@@ -95,12 +94,12 @@ class MulticastTransport(Transport):
 
 
 class MulticastConnection(BaseConnection, Protocol12):
-    def __init__(self, wait_on_receipt=False, encoding='utf-8'):
+    def __init__(self, wait_on_receipt=False, encoding="utf-8"):
         """
         :param bool wait_on_receipt: deprecated, ignored
         """
         self.transport = MulticastTransport(encoding)
-        self.transport.set_listener('mcast-listener', self)
+        self.transport.set_listener("mcast-listener", self)
         self.transactions = {}
         Protocol12.__init__(self, self.transport, (0, 0))
 
@@ -114,7 +113,7 @@ class MulticastConnection(BaseConnection, Protocol12):
         """
         self.transport.start()
 
-    def subscribe(self, destination, id, ack='auto', headers=None, **keyword_headers):
+    def subscribe(self, destination, id, ack="auto", headers=None, **keyword_headers):
         """
         :param str destination:
         :param str id:
@@ -141,7 +140,7 @@ class MulticastConnection(BaseConnection, Protocol12):
         Protocol12.disconnect(self, receipt, headers, **keyword_headers)
         self.transport.stop()
 
-    def send_frame(self, cmd, headers=None, body=''):
+    def send_frame(self, cmd, headers=None, body=""):
         """
         :param str cmd:
         :param dict headers:
@@ -149,30 +148,30 @@ class MulticastConnection(BaseConnection, Protocol12):
         """
         if headers is None:
             headers = {}
-        frame = utils.Frame(cmd, headers, body)
+        frame = Frame(cmd, headers, body)
 
         if cmd == CMD_BEGIN:
             trans = headers[HDR_TRANSACTION]
             if trans in self.transactions:
-                self.notify('error', {}, 'Transaction %s already started' % trans)
+                self.notify("error", Frame(None, {}, "Transaction %s already started" % trans))
             else:
                 self.transactions[trans] = []
         elif cmd == CMD_COMMIT:
             trans = headers[HDR_TRANSACTION]
             if trans not in self.transactions:
-                self.notify('error', {}, 'Transaction %s not started' % trans)
+                self.notify("error", Frame(None, {}, "Transaction %s not started" % trans))
             else:
                 for f in self.transactions[trans]:
                     self.transport.transmit(f)
                 del self.transactions[trans]
         elif cmd == CMD_ABORT:
-            trans = headers['transaction']
+            trans = headers["transaction"]
             del self.transactions[trans]
         else:
-            if 'transaction' in headers:
-                trans = headers['transaction']
+            if "transaction" in headers:
+                trans = headers["transaction"]
                 if trans not in self.transactions:
-                    self.transport.notify('error', {}, 'Transaction %s not started' % trans)
+                    self.transport.notify("error", Frame(None, {}, "Transaction %s not started" % trans))
                     return
                 else:
                     self.transactions[trans].append(frame)
