@@ -5,7 +5,6 @@ Provides connection classes for `1.0 <http://stomp.github.io/stomp-specification
 `1.2 <http://stomp.github.io/stomp-specification-1.2.html>`_ versions of the STOMP protocol.
 """
 
-from stomp.listener import *
 from stomp.protocol import *
 from stomp.transport import *
 from stomp.utils import get_uuid
@@ -25,20 +24,22 @@ class BaseConnection(Publisher):
     def __enter__(self):
         self.disconnect_receipt_id = get_uuid()
         self.disconnect_listener = WaitingListener(self.disconnect_receipt_id)
-        self.set_listener('ZZZZZ-disconnect-listener', self.disconnect_listener)
+        self.set_listener("ZZZZZ-disconnect-listener", self.disconnect_listener)
         return self
 
+    def disconnect(self, receipt=None, headers=None, **keyword_headers): pass
+
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.disconnect(self.disconnect_receipt_id)
         self.disconnect_listener.wait_on_receipt()
         self.disconnect_listener.wait_on_disconnected()
 
-    def set_listener(self, name, lstnr):
+    def set_listener(self, name, listener):
         """
         :param str name:
-        :param ConnectionListener lstnr:
+        :param ConnectionListener listener:
         """
-        self.transport.set_listener(name, lstnr)
+        self.transport.set_listener(name, listener)
 
     def remove_listener(self, name):
         """
@@ -54,18 +55,6 @@ class BaseConnection(Publisher):
         """
         return self.transport.get_listener(name)
 
-    def start(self):
-        """
-        Deprecated. Still included to provide backwards compatibility with previous versions of stomp.py (no longer required)
-        """
-        pass
-
-    def stop(self):
-        """
-        Deprecated. Still included to provide backwards compatibility with previous versions of stomp.py (no longer required)
-        """
-        pass
-
     def is_connected(self):
         """
         :rtype: bool
@@ -78,8 +67,8 @@ class BaseConnection(Publisher):
     def set_ssl(self, *args, **kwargs):
         self.transport.set_ssl(*args, **kwargs)
 
-    def get_ssl(self, *args, **kwargs):
-        self.transport.get_ssl(*args, **kwargs)
+    def get_ssl(self, host_and_port=None):
+        return self.transport.get_ssl(host_and_port)
 
 
 class StompConnection10(BaseConnection, Protocol10):
@@ -105,14 +94,14 @@ class StompConnection10(BaseConnection, Protocol10):
                  timeout=None,
                  keepalive=None,
                  auto_decode=True,
-                 encoding='utf-8',
+                 encoding="utf-8",
                  auto_content_length=True,
-                 recv_bytes=1024):
+                 bind_host_port=None):
         transport = Transport(host_and_ports, prefer_localhost, try_loopback_connect,
                               reconnect_sleep_initial, reconnect_sleep_increase, reconnect_sleep_jitter,
                               reconnect_sleep_max, reconnect_attempts_max, use_ssl, ssl_key_file, ssl_cert_file,
                               ssl_ca_certs, ssl_cert_validator, ssl_version, timeout,
-                              keepalive, None, auto_decode, encoding)
+                              keepalive, None, auto_decode, encoding, bind_host_port=bind_host_port)
         BaseConnection.__init__(self, transport)
         Protocol10.__init__(self, transport, auto_content_length)
 
@@ -158,17 +147,18 @@ class StompConnection11(BaseConnection, Protocol11):
                  keepalive=None,
                  vhost=None,
                  auto_decode=True,
-                 encoding='utf-8',
+                 encoding="utf-8",
                  auto_content_length=True,
                  heart_beat_receive_scale=1.5,
-                 recv_byte=1024):
+                 bind_host_port=None):
         transport = Transport(host_and_ports, prefer_localhost, try_loopback_connect,
                               reconnect_sleep_initial, reconnect_sleep_increase, reconnect_sleep_jitter,
                               reconnect_sleep_max, reconnect_attempts_max, use_ssl, ssl_key_file, ssl_cert_file,
                               ssl_ca_certs, ssl_cert_validator, ssl_version, timeout,
-                              keepalive, vhost, auto_decode, encoding)
+                              keepalive, vhost, auto_decode, encoding, bind_host_port=bind_host_port)
         BaseConnection.__init__(self, transport)
-        Protocol11.__init__(self, transport, heartbeats, auto_content_length, heart_beat_receive_scale=heart_beat_receive_scale)
+        Protocol11.__init__(self, transport, heartbeats, auto_content_length,
+                            heart_beat_receive_scale=heart_beat_receive_scale)
 
     def connect(self, *args, **kwargs):
         self.transport.start()
@@ -212,17 +202,18 @@ class StompConnection12(BaseConnection, Protocol12):
                  keepalive=None,
                  vhost=None,
                  auto_decode=True,
-                 encoding='utf-8',
+                 encoding="utf-8",
                  auto_content_length=True,
                  heart_beat_receive_scale=1.5,
-                 recv_bytes=1024):
+                 bind_host_port=None):
         transport = Transport(host_and_ports, prefer_localhost, try_loopback_connect,
                               reconnect_sleep_initial, reconnect_sleep_increase, reconnect_sleep_jitter,
                               reconnect_sleep_max, reconnect_attempts_max, use_ssl, ssl_key_file, ssl_cert_file,
                               ssl_ca_certs, ssl_cert_validator, ssl_version, timeout,
-                              keepalive, vhost, auto_decode, encoding)
+                              keepalive, vhost, auto_decode, encoding, bind_host_port=bind_host_port)
         BaseConnection.__init__(self, transport)
-        Protocol12.__init__(self, transport, heartbeats, auto_content_length, heart_beat_receive_scale=heart_beat_receive_scale)
+        Protocol12.__init__(self, transport, heartbeats, auto_content_length,
+                            heart_beat_receive_scale=heart_beat_receive_scale)
 
     def connect(self, *args, **kwargs):
         self.transport.start()
@@ -239,3 +230,7 @@ class StompConnection12(BaseConnection, Protocol12):
         Protocol12.disconnect(self, receipt, headers, **keyword_headers)
         if receipt is not None:
             self.transport.stop()
+
+    @staticmethod
+    def is_eol(c):
+        return c == b"\x0a" or c == b"\x0d\x0a"
