@@ -1,13 +1,8 @@
 """Provides the 1.0, 1.1 and 1.2 protocol classes.
 """
 
-from stomp.constants import *
 from stomp.exception import ConnectFailedException
 from stomp.listener import *
-import stomp.utils as utils
-
-
-log = logging.getLogger('stomp.py')
 
 
 class Protocol10(ConnectionListener):
@@ -17,15 +12,16 @@ class Protocol10(ConnectionListener):
     Most users should not instantiate the protocol directly. See :py:mod:`stomp.connect` for connection classes.
 
     :param transport:
-    :param bool auto_content_length: Whether to calculate and send the content-length header automatically if it has not been set
+    :param bool auto_content_length: Whether to calculate and send the content-length header
+    automatically if it has not been set
     """
     def __init__(self, transport, auto_content_length=True):
         self.transport = transport
         self.auto_content_length = auto_content_length
-        transport.set_listener('protocol-listener', self)
-        self.version = '1.0'
+        transport.set_listener("protocol-listener", self)
+        self.version = "1.0"
 
-    def send_frame(self, cmd, headers=None, body=''):
+    def send_frame(self, cmd, headers=None, body=""):
         """
         Encode and send a stomp frame
         through the underlying transport.
@@ -56,6 +52,7 @@ class Protocol10(ConnectionListener):
 
         :param str id: identifier of the message
         :param str transaction: include the acknowledgement in the specified transaction
+        :param str receipt: the receipt id
         """
         assert id is not None, "'id' is required"
         headers = {HDR_MESSAGE_ID: id}
@@ -135,7 +132,7 @@ class Protocol10(ConnectionListener):
         :param keyword_headers: any additional headers the broker requires
         """
         if not self.transport.is_connected():
-            log.debug('Not sending disconnect, already disconnected')
+            logging.debug("Not sending disconnect, already disconnected")
             return
         headers = utils.merge_headers([headers, keyword_headers])
         rec = receipt or utils.get_uuid()
@@ -163,7 +160,7 @@ class Protocol10(ConnectionListener):
             headers[HDR_CONTENT_LENGTH] = len(body)
         self.send_frame(CMD_SEND, headers, body)
 
-    def subscribe(self, destination, id=None, ack='auto', headers=None, **keyword_headers):
+    def subscribe(self, destination, id=None, ack="auto", headers=None, **keyword_headers):
         """
         Subscribe to a destination.
 
@@ -209,16 +206,17 @@ class Protocol11(HeartbeatListener, ConnectionListener):
 
     :param transport:
     :param (int,int) heartbeats:
-    :param bool auto_content_length: Whether to calculate and send the content-length header automatically if it has not been set
-    :param float heart_beat_receive_scale: how long to wait for a heartbeat before timing out, as a scale factor of receive time
+    :param bool auto_content_length: Whether to calculate and send the content-length header
+    automatically if it has not been set
+    :param float heart_beat_receive_scale: how long to wait for a heartbeat before timing out,
+    as a scale factor of receive time
     """
     def __init__(self, transport, heartbeats=(0, 0), auto_content_length=True, heart_beat_receive_scale=1.5):
-        HeartbeatListener.__init__(self, heartbeats)
+        HeartbeatListener.__init__(self, transport, heartbeats, heart_beat_receive_scale)
         self.transport = transport
         self.auto_content_length = auto_content_length
-        transport.set_listener('protocol-listener', self)
-        self.version = '1.1'
-        self.heart_beat_receive_scale = heart_beat_receive_scale
+        transport.set_listener("protocol-listener", self)
+        self.version = "1.1"
 
     def _escape_headers(self, headers):
         """
@@ -226,12 +224,12 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         """
         for key, val in headers.items():
             try:
-                val = val.replace('\\', '\\\\').replace('\n', '\\n').replace(':', '\\c')
+                val = val.replace("\\", "\\\\").replace("\n", "\\n").replace(":", "\\c")
             except:
                 pass
             headers[key] = val
 
-    def send_frame(self, cmd, headers=None, body=''):
+    def send_frame(self, cmd, headers=None, body=""):
         """
         Encode and send a stomp frame
         through the underlying transport:
@@ -267,6 +265,7 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         :param str id: identifier of the message
         :param str subscription: the subscription this message is associated with
         :param str transaction: include the acknowledgement in the specified transaction
+        :param str receipt: the receipt id
         """
         assert id is not None, "'id' is required"
         assert subscription is not None, "'subscription' is required"
@@ -350,7 +349,7 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         :param keyword_headers: any additional headers the broker requires
         """
         if not self.transport.is_connected():
-            log.debug('Not sending disconnect, already disconnected')
+            logging.debug("Not sending disconnect, already disconnected")
             return
         headers = utils.merge_headers([headers, keyword_headers])
         rec = receipt or utils.get_uuid()
@@ -358,17 +357,20 @@ class Protocol11(HeartbeatListener, ConnectionListener):
         self.set_receipt(rec, CMD_DISCONNECT)
         self.send_frame(CMD_DISCONNECT, headers)
 
-    def nack(self, id, subscription, transaction=None, receipt=None):
+    def nack(self, id, subscription, transaction=None, receipt=None, **keyword_headers):
         """
         Let the server know that a message was not consumed.
 
         :param str id: the unique id of the message to nack
         :param str subscription: the subscription this message is associated with
         :param str transaction: include this nack in a named transaction
+        :param str receipt: the receipt id
+        :param keyword_headers: any additional headers to send with the nack command
         """
         assert id is not None, "'id' is required"
         assert subscription is not None, "'subscription' is required"
         headers = {HDR_MESSAGE_ID: id, HDR_SUBSCRIPTION: subscription}
+        headers = utils.merge_headers([headers, keyword_headers])
         if transaction:
             headers[HDR_TRANSACTION] = transaction
         if receipt:
@@ -377,9 +379,11 @@ class Protocol11(HeartbeatListener, ConnectionListener):
 
     def send(self, destination, body, content_type=None, headers=None, **keyword_headers):
         """
-        Send a message to a destination in the messaging system (as per https://stomp.github.io/stomp-specification-1.2.html#SEND)
+        Send a message to a destination in the messaging system (as per
+         https://stomp.github.io/stomp-specification-1.2.html#SEND)
 
-        :param str destination: the destination (such as a message queue - for example '/queue/test' - or a message topic)
+        :param str destination: the destination (such as a message queue - for example
+        '/queue/test' - or a message topic)
         :param body: the content of the message
         :param str content_type: the MIME type of message
         :param dict headers: additional headers to send in the message frame
@@ -395,13 +399,14 @@ class Protocol11(HeartbeatListener, ConnectionListener):
             headers[HDR_CONTENT_LENGTH] = len(body)
         self.send_frame(CMD_SEND, headers, body)
 
-    def subscribe(self, destination, id, ack='auto', headers=None, **keyword_headers):
+    def subscribe(self, destination, id, ack="auto", headers=None, **keyword_headers):
         """
         Subscribe to a destination
 
         :param str destination: the topic or queue to subscribe to
         :param str id: the identifier to uniquely identify the subscription
-        :param str ack: either auto, client or client-individual (see https://stomp.github.io/stomp-specification-1.2.html#SUBSCRIBE for more info)
+        :param str ack: either auto, client or client-individual
+        (see https://stomp.github.io/stomp-specification-1.2.html#SUBSCRIBE for more info)
         :param dict headers: a map of any additional headers to send with the subscription
         :param keyword_headers: any additional headers to send with the subscription
         """
@@ -435,12 +440,15 @@ class Protocol12(Protocol11):
 
     :param transport:
     :param (int,int) heartbeats:
-    :param bool auto_content_length: Whether to calculate and send the content-length header automatically if it has not been set
-    :param float heart_beat_receive_scale: how long to wait for a heartbeat before timing out, as a scale factor of receive time
+    :param bool auto_content_length: Whether to calculate and send the content-length header
+    automatically if it has not been set
+    :param float heart_beat_receive_scale: how long to wait for a heartbeat before timing out,
+    as a scale factor of receive time
     """
     def __init__(self, transport, heartbeats=(0, 0), auto_content_length=True, heart_beat_receive_scale=1.5):
-        Protocol11.__init__(self, transport, heartbeats, auto_content_length, heart_beat_receive_scale=heart_beat_receive_scale)
-        self.version = '1.2'
+        Protocol11.__init__(self, transport, heartbeats, auto_content_length,
+                            heart_beat_receive_scale=heart_beat_receive_scale)
+        self.version = "1.2"
 
     def _escape_headers(self, headers):
         """
@@ -448,7 +456,7 @@ class Protocol12(Protocol11):
         """
         for key, val in headers.items():
             try:
-                val = val.replace('\\', '\\\\').replace('\n', '\\n').replace(':', '\\c').replace('\r', '\\r')
+                val = val.replace("\\", "\\\\").replace("\n", "\\n").replace(":", "\\c").replace("\r", "\\r")
             except:
                 pass
             headers[key] = val
@@ -459,6 +467,7 @@ class Protocol12(Protocol11):
 
         :param str id: identifier of the message
         :param str transaction: include the acknowledgement in the specified transaction
+        :param str receipt: the receipt id
         """
         assert id is not None, "'id' is required"
         headers = {HDR_ID: id}
@@ -468,15 +477,18 @@ class Protocol12(Protocol11):
             headers[HDR_RECEIPT] = receipt
         self.send_frame(CMD_ACK, headers)
 
-    def nack(self, id, transaction=None, receipt=None):
+    def nack(self, id, transaction=None, receipt=None, **keyword_headers):
         """
         Let the server know that a message was not consumed.
 
         :param str id: the unique id of the message to nack
         :param str transaction: include this nack in a named transaction
+        :param str receipt: the receipt id
+        :param keyword_headers: any additional headers to send with the nack command
         """
         assert id is not None, "'id' is required"
         headers = {HDR_ID: id}
+        headers = utils.merge_headers([headers, keyword_headers])
         if transaction:
             headers[HDR_TRANSACTION] = transaction
         if receipt:
