@@ -1,29 +1,29 @@
-"""Provides the underlying transport functionality (for stomp message transmission) - (mostly) independent from the actual STOMP protocol
+"""Provides the underlying transport functionality (for stomp message transmission) - (mostly) independent
+from the actual STOMP protocol
 """
 
 import errno
-from io import BytesIO
-import logging
 import math
 import random
-import re
-import socket
 import sys
-import threading
 import time
 import warnings
+from io import BytesIO
+from time import monotonic
 
 try:
     import ssl
     from ssl import SSLError
 
     DEFAULT_SSL_VERSION = ssl.PROTOCOL_TLSv1
-except (ImportError, AttributeError):  # python version < 2.6 without the backported ssl module
+except (ImportError, AttributeError):
     ssl = None
 
+
     class SSLError(object):
         pass
 
+
     DEFAULT_SSL_VERSION = None
 
 try:
@@ -34,15 +34,10 @@ try:
 except ImportError:
     LINUX_KEEPALIVE_AVAIL = False
 
-from stomp.backward import decode, encode, get_errno, monotonic, pack
-from stomp.backwardsock import get_socket
-from stomp.constants import *
 import stomp.exception as exception
 import stomp.listener
-import stomp.utils as utils
-
-
-log = logging.getLogger('stomp.py')
+from stomp.utils import *
+from stomp import logging
 
 
 class BaseTransport(stomp.listener.Publisher):
@@ -60,24 +55,25 @@ class BaseTransport(stomp.listener.Publisher):
     #
     # Used to parse the STOMP "content-length" header lines,
     #
-    __content_length_re = re.compile(b'^content-length[:]\\s*(?P<value>[0-9]+)', re.MULTILINE)
+    __content_length_re = re.compile(b"^content-length[:]\\s*(?P<value>[0-9]+)", re.MULTILINE)
 
-    def __init__(self, auto_decode=True, encoding='utf-8'):
-        self.__recvbuf = b''
+    def __init__(self, auto_decode=True, encoding="utf-8", is_eol_fc=is_eol_default):
+        self.__recvbuf = b""
         self.listeners = {}
         self.running = False
         self.blocking = None
         self.connected = False
         self.connection_error = False
+        self.disconnecting = False
         self.__receipts = {}
         self.current_host_and_port = None
-
+        self.bind_host_port = None
         # flag used when we receive the disconnect receipt
         self.__disconnect_receipt = None
-        self.__notified_on_disconnect = False
+        self.notified_on_disconnect = False
 
         # function for creating threads used by the connection
-        self.create_thread_fc = utils.default_create_thread
+        self.create_thread_fc = default_create_thread
 
         self.__listeners_change_condition = threading.Condition()
         self.__receiver_thread_exit_condition = threading.Condition()
@@ -86,6 +82,7 @@ class BaseTransport(stomp.listener.Publisher):
         self.__connect_wait_condition = threading.Condition()
         self.__auto_decode = auto_decode
         self.__encoding = encoding
+        self.__is_eol = is_eol_fc
 
     def override_threading(self, create_thread_fc):
         """
@@ -105,13 +102,14 @@ class BaseTransport(stomp.listener.Publisher):
         """
         Start the connection. This should be called after all
         listeners have been registered. If this method is not called,
-        no frames will be received by the connection.
+        no frames will be received by the connection and no SSL/TLS
+        handshake will occur.
         """
         self.running = True
         self.attempt_connection()
         receiver_thread = self.create_thread_fc(self.__receiver_loop)
-        receiver_thread.name = "StompReceiver%s" % getattr(receiver_thread, "name", "Thread")
-        self.notify('connecting')
+        logging.info("Created thread %s using func %s", receiver_thread, self.create_thread_fc)
+        self.notify("connecting")
 
     def stop(self):
         """
@@ -135,6 +133,7 @@ class BaseTransport(stomp.listener.Publisher):
         with self.__connect_wait_condition:
             self.connected = connected
             if connected:
+                self.disconnecting = False
                 self.__connect_wait_condition.notify()
 
     def set_receipt(self, receipt_id, value):
@@ -155,6 +154,7 @@ class BaseTransport(stomp.listener.Publisher):
         :param str name: the name of the listener
         :param ConnectionListener listener: the listener object
         """
+        assert listener is not None
         with self.__listeners_change_condition:
             self.listeners[name] = listener
 
@@ -183,28 +183,27 @@ class BaseTransport(stomp.listener.Publisher):
         :param bytes frame_str: raw frame content
         """
         frame_type = f.cmd.lower()
-        if frame_type in ['connected', 'message', 'receipt', 'error', 'heartbeat']:
-            if frame_type == 'message':
-                (f.headers, f.body) = self.notify('before_message', f.headers, f.body)
-            if log.isEnabledFor(logging.DEBUG):
-                log.debug("Received frame: %r, headers=%r, body=%r", f.cmd, f.headers, f.body)
+        if frame_type in ["connected", "message", "receipt", "error", "heartbeat"]:
+            if frame_type == "message":
+                self.notify("before_message", f)
+            if logging.isEnabledFor(logging.DEBUG):
+                logging.debug("Received frame: %r, headers=%r, body=%r", f.cmd, f.headers, f.body)
             else:
-                log.info("Received frame: %r, len(body)=%r", f.cmd, utils.length(f.body))
-            self.notify(frame_type, f.headers, f.body)
+                logging.info("Received frame: %r, len(body)=%r", f.cmd, length(f.body))
+            self.notify(frame_type, f)
         else:
-            log.warning("Unknown response frame type: '%s' (frame length was %d)", frame_type, utils.length(frame_str))
+            logging.warning("Unknown response frame type: '%s' (frame length was %d)", frame_type, length(frame_str))
 
-    def notify(self, frame_type, headers=None, body=None):
+    def notify(self, frame_type, frame=None):
         """
         Utility function for notifying listeners of incoming and outgoing messages
 
         :param str frame_type: the type of message
-        :param dict headers: the map of headers associated with the message
-        :param body: the content of the message
+        :param Frame frame: the stomp frame
         """
-        if frame_type == 'receipt':
+        if frame_type == "receipt":
             # logic for wait-on-receipt notification
-            receipt = headers['receipt-id']
+            receipt = frame.headers["receipt-id"]
             receipt_value = self.__receipts.get(receipt)
             with self.__send_wait_condition:
                 self.set_receipt(receipt, None)
@@ -217,40 +216,34 @@ class BaseTransport(stomp.listener.Publisher):
                     self.disconnect_socket()
                 self.__disconnect_receipt = None
 
-        elif frame_type == 'connected':
+        elif frame_type == "connected":
             self.set_connected(True)
 
-        elif frame_type == 'disconnected':
-            self.__notified_on_disconnect = True
+        elif frame_type == "disconnected":
+            self.notified_on_disconnect = True
             self.set_connected(False)
 
         with self.__listeners_change_condition:
             listeners = sorted(self.listeners.items())
 
         for (_, listener) in listeners:
-            if not listener:
-                continue
-
-            notify_func = getattr(listener, 'on_%s' % frame_type, None)
+            notify_func = getattr(listener, "on_%s" % frame_type, None)
             if not notify_func:
-                log.debug("listener %s has no method on_%s", listener, frame_type)
+                logging.debug("listener %s has no method on_%s", listener, frame_type)
                 continue
-            if frame_type in ('heartbeat', 'disconnected'):
+            if frame_type in ("heartbeat", "disconnected"):
                 notify_func()
                 continue
-            if frame_type == 'connecting':
+            if frame_type == "connecting":
                 notify_func(self.current_host_and_port)
                 continue
 
-            if frame_type == 'error' and not self.connected:
+            if frame_type == "error" and not self.connected:
                 with self.__connect_wait_condition:
                     self.connection_error = True
                     self.__connect_wait_condition.notify()
 
-            rtn = notify_func(headers, body)
-            if rtn:
-                (headers, body) = rtn
-        return (headers, body)
+            notify_func(frame)
 
     def transmit(self, frame):
         """
@@ -262,8 +255,6 @@ class BaseTransport(stomp.listener.Publisher):
             listeners = sorted(self.listeners.items())
 
         for (_, listener) in listeners:
-            if not listener:
-                continue
             try:
                 listener.on_send(frame)
             except AttributeError:
@@ -272,13 +263,13 @@ class BaseTransport(stomp.listener.Publisher):
         if frame.cmd == CMD_DISCONNECT and HDR_RECEIPT in frame.headers:
             self.__disconnect_receipt = frame.headers[HDR_RECEIPT]
 
-        lines = utils.convert_frame(frame)
+        lines = convert_frame(frame)
         packed_frame = pack(lines)
 
-        if log.isEnabledFor(logging.DEBUG):
-            log.debug("Sending frame: %s", utils.clean_lines(lines))
+        if logging.isEnabledFor(logging.DEBUG):
+            logging.debug("Sending frame: %s", clean_lines(lines))
         else:
-            log.info("Sending frame: %r", frame.cmd or "heartbeat")
+            logging.info("Sending frame: %r", frame.cmd or "heartbeat")
         self.send(packed_frame)
 
     def send(self, encoded_frame):
@@ -313,6 +304,7 @@ class BaseTransport(stomp.listener.Publisher):
         """
         Disconnect the socket.
         """
+        pass
 
     def wait_for_connection(self, timeout=None):
         """
@@ -334,7 +326,7 @@ class BaseTransport(stomp.listener.Publisher):
         """
         Main loop listening for incoming data.
         """
-        log.info("Starting receiver loop")
+        logging.info("Starting receiver loop (%s)", threading.current_thread())
         notify_disconnected = True
         try:
             while self.running:
@@ -343,7 +335,10 @@ class BaseTransport(stomp.listener.Publisher):
                         frames = self.__read()
 
                         for frame in frames:
-                            f = utils.parse_frame(frame)
+                            if self.__is_eol(frame):
+                                f = HEARTBEAT_FRAME
+                            else:
+                                f = parse_frame(frame)
                             if f is None:
                                 continue
                             if self.__auto_decode:
@@ -354,7 +349,7 @@ class BaseTransport(stomp.listener.Publisher):
                         #
                         # Clear out any half-received messages after losing connection
                         #
-                        self.__recvbuf = b''
+                        self.__recvbuf = b""
                         self.running = False
                         notify_disconnected = True
                     break
@@ -364,12 +359,13 @@ class BaseTransport(stomp.listener.Publisher):
             with self.__receiver_thread_exit_condition:
                 self.__receiver_thread_exited = True
                 self.__receiver_thread_exit_condition.notifyAll()
-            log.info("Receiver loop ended")
-            self.notify('receiver_loop_completed')
-            if notify_disconnected:
-                self.notify('disconnected')
+            logging.info("Receiver loop ended")
+            self.notify("receiver_loop_completed")
+            if notify_disconnected and not self.notified_on_disconnect:
+                self.notify("disconnected")
             with self.__connect_wait_condition:
                 self.__connect_wait_condition.notifyAll()
+            self.notified_on_disconnect = False
 
     def __read(self):
         """
@@ -384,14 +380,15 @@ class BaseTransport(stomp.listener.Publisher):
                 try:
                     c = self.receive()
                 except exception.InterruptedException:
-                    log.debug("socket read interrupted, restarting")
+                    logging.debug("socket read interrupted, restarting")
                     continue
             except Exception:
-                log.debug("socket read error", exc_info=True)
-                c = b''
+                logging.debug("socket read error", exc_info=logging.verbose)
+                c = b""
             if c is None or len(c) == 0:
+                logging.debug("nothing received, raising CCE")
                 raise exception.ConnectionClosedException()
-            if c == b'\x0a' and not self.__recvbuf and not fastbuf.tell():
+            if self.__is_eol(c) and not self.__recvbuf and not fastbuf.tell():
                 #
                 # EOL to an empty receive buffer: treat as heartbeat.
                 # Note that this may misdetect an optional EOL at end of frame as heartbeat in case the
@@ -401,7 +398,7 @@ class BaseTransport(stomp.listener.Publisher):
                 fastbuf.close()
                 return [c]
             fastbuf.write(c)
-            if b'\x00' in c:
+            if b"\x00" in c:
                 #
                 # Possible end of frame
                 #
@@ -412,16 +409,16 @@ class BaseTransport(stomp.listener.Publisher):
 
         if self.__recvbuf and self.running:
             while True:
-                pos = self.__recvbuf.find(b'\x00')
+                pos = self.__recvbuf.find(b"\x00")
 
                 if pos >= 0:
                     frame = self.__recvbuf[0:pos]
-                    preamble_end_match = utils.PREAMBLE_END_RE.search(frame)
+                    preamble_end_match = PREAMBLE_END_RE.search(frame)
                     if preamble_end_match:
                         preamble_end = preamble_end_match.start()
                         content_length_match = BaseTransport.__content_length_re.search(frame[0:preamble_end])
                         if content_length_match:
-                            content_length = int(content_length_match.group('value'))
+                            content_length = int(content_length_match.group("value"))
                             content_offset = preamble_end_match.end()
                             frame_size = content_offset + content_length
                             if frame_size > len(frame):
@@ -441,8 +438,13 @@ class BaseTransport(stomp.listener.Publisher):
                     #
                     # Ignore optional EOLs at end of frame
                     #
-                    while self.__recvbuf[pos:pos + 1] == b'\x0a':
-                        pos += 1
+                    while 1:
+                        if self.__is_eol(self.__recvbuf[pos:pos + 1]):
+                            pos += 1
+                        elif self.__is_eol(self.__recvbuf[pos:pos + 2]):
+                            pos += 2
+                        else:
+                            break
                     self.__recvbuf = self.__recvbuf[pos:]
                 else:
                     break
@@ -513,13 +515,15 @@ class Transport(BaseTransport):
                  keepalive=None,
                  vhost=None,
                  auto_decode=True,
-                 encoding='utf-8',
-                 recv_bytes=1024
-                 ):
-        BaseTransport.__init__(self, auto_decode, encoding)
+                 encoding="utf-8",
+                 recv_bytes=1024,
+                 is_eol_fc=is_eol_default,
+                 bind_host_port=None):
+        BaseTransport.__init__(self, auto_decode, encoding, is_eol_fc)
 
         if host_and_ports is None:
-            host_and_ports = [('localhost', 61613)]
+            logging.debug("No hosts_and_ports specified, adding default localhost")
+            host_and_ports = [("localhost", 61613)]
 
         sorted_host_and_ports = []
         sorted_host_and_ports.extend(host_and_ports)
@@ -529,7 +533,7 @@ class Transport(BaseTransport):
         # the list
         #
         if prefer_localhost:
-            sorted_host_and_ports.sort(key=utils.is_localhost)
+            sorted_host_and_ports.sort(key=is_localhost)
 
         #
         # If the user wishes to attempt connecting to local ports using the loopback interface, for each (host, port)
@@ -539,9 +543,10 @@ class Transport(BaseTransport):
         loopback_host_and_ports = []
         if try_loopback_connect:
             for host_and_port in sorted_host_and_ports:
-                if utils.is_localhost(host_and_port) == 1:
+                if is_localhost(host_and_port) == 1:
                     port = host_and_port[1]
-                    if not (("127.0.0.1", port) in sorted_host_and_ports or ("localhost", port) in sorted_host_and_ports):
+                    if not (("127.0.0.1", port) in sorted_host_and_ports or (
+                    "localhost", port) in sorted_host_and_ports):
                         loopback_host_and_ports.append(("127.0.0.1", port))
 
         #
@@ -550,6 +555,7 @@ class Transport(BaseTransport):
         self.__host_and_ports = []
         self.__host_and_ports.extend(loopback_host_and_ports)
         self.__host_and_ports.extend(sorted_host_and_ports)
+        self.__bind_host_port = bind_host_port
 
         self.__reconnect_sleep_initial = reconnect_sleep_initial
         self.__reconnect_sleep_increase = reconnect_sleep_increase
@@ -606,15 +612,15 @@ class Transport(BaseTransport):
                     # unwrap seems flaky on Win with the back-ported ssl mod, so catch any exception and log it
                     #
                     _, e, _ = sys.exc_info()
-                    log.warning(e)
-            elif hasattr(socket, 'SHUT_RDWR'):
+                    logging.warning(e)
+            elif hasattr(socket, "SHUT_RDWR"):
                 try:
                     self.socket.shutdown(socket.SHUT_RDWR)
                 except socket.error:
                     _, e, _ = sys.exc_info()
                     # ignore when socket already closed
                     if get_errno(e) != errno.ENOTCONN:
-                        log.warning("Unable to issue SHUT_RDWR on socket because of error '%s'", e)
+                        logging.warning("Unable to issue SHUT_RDWR on socket because of error '%s'", e)
 
         #
         # split this into a separate check, because sometimes the socket is nulled between shutdown and this call
@@ -624,10 +630,11 @@ class Transport(BaseTransport):
                 self.socket.close()
             except socket.error:
                 _, e, _ = sys.exc_info()
-                log.warning("Unable to close socket because of error '%s'", e)
+                logging.warning("Unable to close socket because of error '%s'", e)
         self.current_host_and_port = None
         self.socket = None
-        self.notify('disconnected')
+        if not self.notified_on_disconnect:
+            self.notify("disconnected")
 
     def send(self, encoded_frame):
         """
@@ -639,7 +646,7 @@ class Transport(BaseTransport):
                     self.socket.sendall(encoded_frame)
             except Exception:
                 _, e, _ = sys.exc_info()
-                log.error("Error sending frame", exc_info=1)
+                logging.error("Error sending frame", exc_info=True)
                 raise e
         else:
             raise exception.NotConnectedException()
@@ -653,7 +660,7 @@ class Transport(BaseTransport):
         except socket.error:
             _, e, _ = sys.exc_info()
             if get_errno(e) in (errno.EAGAIN, errno.EINTR):
-                log.debug("socket read interrupted, restarting")
+                logging.debug("socket read interrupted, restarting")
                 raise exception.InterruptedException()
             if self.is_connected():
                 raise
@@ -667,7 +674,6 @@ class Transport(BaseTransport):
         except:
             pass  # ignore errors when attempting to close socket
         self.socket = None
-        self.current_host_and_port = None
 
     def __enable_keepalive(self):
         def try_setsockopt(sock, name, fam, opt, val):
@@ -675,9 +681,9 @@ class Transport(BaseTransport):
                 return True  # no value to set always works
             try:
                 sock.setsockopt(fam, opt, val)
-                log.info("keepalive: set %r option to %r on socket", name, val)
+                logging.info("keepalive: set %r option to %r on socket", name, val)
             except:
-                log.error("keepalive: unable to set %r option to %r on socket", name, val)
+                logging.error("keepalive: unable to set %r option to %r on socket", name, val)
                 return False
             return True
 
@@ -687,37 +693,37 @@ class Transport(BaseTransport):
             return
 
         if ka is True:
-            ka_sig = 'auto'
+            ka_sig = "auto"
             ka_args = ()
         else:
             try:
                 ka_sig = ka[0]
                 ka_args = ka[1:]
             except Exception:
-                log.error("keepalive: bad specification %r", ka)
+                logging.error("keepalive: bad specification %r", ka)
                 return
 
-        if ka_sig == 'auto':
+        if ka_sig == "auto":
             if LINUX_KEEPALIVE_AVAIL:
-                ka_sig = 'linux'
+                ka_sig = "linux"
                 ka_args = None
-                log.info("keepalive: autodetected linux-style support")
+                logging.info("keepalive: autodetected linux-style support")
             else:
-                log.error("keepalive: unable to detect any implementation, DISABLED!")
+                logging.error("keepalive: unable to detect any implementation, DISABLED!")
                 return
 
-        if ka_sig == 'linux':
-            log.info("keepalive: activating linux-style support")
+        if ka_sig == "linux":
+            logging.info("keepalive: activating linux-style support")
             if ka_args is None:
-                log.info("keepalive: using system defaults")
+                logging.info("keepalive: using system defaults")
                 ka_args = (None, None, None)
             lka_idle, lka_intvl, lka_cnt = ka_args
-            if try_setsockopt(self.socket, 'enable', SOL_SOCKET, SO_KEEPALIVE, 1):
-                try_setsockopt(self.socket, 'idle time', SOL_TCP, TCP_KEEPIDLE, lka_idle)
-                try_setsockopt(self.socket, 'interval', SOL_TCP, TCP_KEEPINTVL, lka_intvl)
-                try_setsockopt(self.socket, 'count', SOL_TCP, TCP_KEEPCNT, lka_cnt)
+            if try_setsockopt(self.socket, "enable", SOL_SOCKET, SO_KEEPALIVE, 1):
+                try_setsockopt(self.socket, "idle time", SOL_TCP, TCP_KEEPIDLE, lka_idle)
+                try_setsockopt(self.socket, "interval", SOL_TCP, TCP_KEEPINTVL, lka_intvl)
+                try_setsockopt(self.socket, "count", SOL_TCP, TCP_KEEPCNT, lka_cnt)
         else:
-            log.error("keepalive: implementation %r not recognized or not supported", ka_sig)
+            logging.error("keepalive: implementation %r not recognized or not supported", ka_sig)
 
     def attempt_connection(self):
         """
@@ -727,31 +733,34 @@ class Transport(BaseTransport):
         sleep_exp = 1
         connect_count = 0
 
-        while self.running and self.socket is None and (
-            connect_count < self.__reconnect_attempts_max or
-            self.__reconnect_attempts_max == -1 ):
+        logging.info("attempt reconnection (%s, %s, %s)", self.running, self.socket, connect_count)
+        while self.running and self.socket is None and (connect_count < self.__reconnect_attempts_max or
+                                                        self.__reconnect_attempts_max == -1):
             for host_and_port in self.__host_and_ports:
                 try:
-                    log.info("Attempting connection to host %s, port %s", host_and_port[0], host_and_port[1])
-                    self.socket = get_socket(host_and_port[0], host_and_port[1], self.__timeout)
+                    logging.info("Attempting connection to host %s, port %s", host_and_port[0], host_and_port[1])
+                    if self.__bind_host_port:
+                        self.socket = socket.create_connection(host_and_port, self.__timeout, self.__bind_host_port)
+                    else:
+                        self.socket = socket.create_connection(host_and_port, self.__timeout)
                     self.__enable_keepalive()
                     need_ssl = self.__need_ssl(host_and_port)
 
                     if need_ssl:  # wrap socket
                         ssl_params = self.get_ssl(host_and_port)
-                        if ssl_params['ca_certs']:
+                        if ssl_params["ca_certs"]:
                             cert_validation = ssl.CERT_REQUIRED
                         else:
                             cert_validation = ssl.CERT_NONE
                         try:
-                            tls_context = ssl.create_default_context(cafile=ssl_params['ca_certs'])
+                            tls_context = ssl.create_default_context(cafile=ssl_params["ca_certs"])
                         except AttributeError:
                             tls_context = None
                         if tls_context:
                             # Wrap the socket for TLS
-                            certfile = ssl_params['cert_file']
-                            keyfile = ssl_params['key_file']
-                            password = ssl_params.get('password')
+                            certfile = ssl_params["cert_file"]
+                            keyfile = ssl_params["key_file"]
+                            password = ssl_params.get("password")
                             if certfile and not keyfile:
                                 keyfile = certfile
                             if certfile:
@@ -764,11 +773,11 @@ class Transport(BaseTransport):
                             # Old-style wrap_socket where we don't have a modern SSLContext (so no SNI)
                             self.socket = ssl.wrap_socket(
                                 self.socket,
-                                keyfile=ssl_params['key_file'],
-                                certfile=ssl_params['cert_file'],
+                                keyfile=ssl_params["key_file"],
+                                certfile=ssl_params["cert_file"],
                                 cert_reqs=cert_validation,
-                                ca_certs=ssl_params['ca_certs'],
-                                ssl_version=ssl_params['ssl_version'])
+                                ca_certs=ssl_params["ca_certs"],
+                                ssl_version=ssl_params["ssl_version"])
 
                     self.socket.settimeout(self.__timeout)
 
@@ -778,19 +787,20 @@ class Transport(BaseTransport):
                     #
                     # Validate server cert
                     #
-                    if need_ssl and ssl_params['cert_validator']:
+                    if need_ssl and ssl_params["cert_validator"]:
                         cert = self.socket.getpeercert()
-                        (ok, errmsg) = ssl_params['cert_validator'](cert, host_and_port[0])
+                        (ok, errmsg) = ssl_params["cert_validator"](cert, host_and_port[0])
                         if not ok:
                             raise SSLError("Server certificate validation failed: %s", errmsg)
 
                     self.current_host_and_port = host_and_port
-                    log.info("Established connection to host %s, port %s", host_and_port[0], host_and_port[1])
+                    logging.info("Established connection to host %s, port %s", host_and_port[0], host_and_port[1])
                     break
                 except socket.error:
                     self.socket = None
                     connect_count += 1
-                    log.warning("Could not connect to host %s, port %s", host_and_port[0], host_and_port[1], exc_info=1)
+                    logging.warning("Could not connect to host %s, port %s", host_and_port[0], host_and_port[1],
+                                    exc_info=logging.verbose)
 
             if self.socket is None:
                 sleep_duration = (min(self.__reconnect_sleep_max,
@@ -798,7 +808,7 @@ class Transport(BaseTransport):
                                        * math.pow(1.0 + self.__reconnect_sleep_increase, sleep_exp)))
                                   * (1.0 + random.random() * self.__reconnect_sleep_jitter))
                 sleep_end = monotonic() + sleep_duration
-                log.debug("Sleeping for %.1f seconds before attempting reconnect", sleep_duration)
+                logging.debug("Sleeping for %.1f seconds before attempting reconnect", sleep_duration)
                 while self.running and monotonic() < sleep_end:
                     time.sleep(0.2)
 
@@ -833,6 +843,7 @@ class Transport(BaseTransport):
                                as returned by ssl.SSLSocket.getpeercert()
         :param ssl_version: SSL protocol to use for the connection. This should be one of the PROTOCOL_x
                             constants provided by the ssl module. The default is ssl.PROTOCOL_TLSv1
+        :param password: SSL password
         """
         if not ssl:
             raise Exception("SSL connection requested, but SSL library not found")
